{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "cy",
  "action.edit": "Golygu'r dudalen hon",
  "action.skip": "Neidio i'r cynnwys",
  "action.view": "Gweld ffynhonnell y dudalen hon",
  "announce.dismiss": "Peidiwch â dangos hyn eto",
  "blog.archive": "Archif",
  "blog.categories": "Categorïau",
  "blog.categories.in": "yn",
  "blog.continue": "Parhau i ddarllen",
  "blog.draft": "Drafft",
  "blog.index": "Nôl i'r mynegai",
  "blog.meta": "Metadata",
  "blog.references": "Dolenni cysylltiedig",
  "clipboard.copy": "Copïo i'r clipfwrdd",
  "clipboard.copied": "Wedi copïo i'r clipfwrdd",
  "consent.accept": "Derbyn",
  "consent.manage": "Rheoli gosodiadau cwcis",
  "consent.reject": "Gwrthod",
  "footer": "Troedyn",
  "footer.next": "Nesaf",
  "footer.previous": "Blaenorol",
  "header": "Pennawd",
  "meta.comments": "Sylwadau",
  "meta.source": "Ffynhonnell",
  "nav": "Llywio",
  "readtime.one": "1 munud i'w ddarllen",
  "readtime.other": "# munud i'w ddarllen",
  "rss.created": "Porthiant RSS",
  "rss.updated": "Porthiant RSS o'r cynnwys wedi'i ddiweddaru",
  "search": "Chwilio",
  "search.placeholder": "Chwilio",
  "search.share": "Rhannu",
  "search.reset": "Clirio",
  "search.result.initializer": "Cychwyn chwilio",
  "search.result.placeholder": "Teipiwch i ddechrau chwilio",
  "search.result.none": "Dim dogfennau sy'n cyfateb",
  "search.result.one": "1 dogfen sy'n cyfateb",
  "search.result.other": "# dogfennau sy'n cyfateb",
  "search.result.more.one": "1 yn rhagor ar y dudalen hon",
  "search.result.more.other": "# yn rhagor ar y dudalen hon",
  "search.result.term.missing": "Ar goll",
  "select.language": "Dewiswch iaith",
  "select.version": "Dewiswch fersiwn",
  "source": "Mynd i'r storfa",
  "source.file.contributors": "Cyfranwyr",
  "source.file.date.created": "Crëwyd",
  "source.file.date.updated": "Diweddariad olaf",
  "tabs": "Tabiau",
  "toc": "Cynnwys tudalen",
  "top": "Nôl i'r brig"
}[key] }}{% endmacro %}
