{#-
  This file was automatically generated - do not edit
-#}
<div class="md-search" data-md-component="search" role="dialog" aria-label="{{ lang.t('search') }}">
  <button type="button" class="md-search__button">
    {{ lang.t('search') }}
  </button>
</div>
