{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "uk",
  "action.edit": "Редагувати сторінку",
  "action.skip": "Перейти до змісту",
  "action.view": "Переглянути вихідний код сторінки",
  "announce.dismiss": "Більше не показувати",
  "blog.archive": "Архівувати",
  "blog.categories": "Категорії",
  "blog.categories.in": "в",
  "blog.continue": "Читати далі",
  "blog.draft": "Чернетка",
  "blog.index": "Повернутись на головну",
  "blog.meta": "Метадані",
  "blog.references": "Пов'язані посилання",
  "clipboard.copy": "Скопіювати в буфер",
  "clipboard.copied": "Скопійовано в буфер",
  "consent.accept": "Прийняти",
  "consent.manage": "Керувати налаштуваннями",
  "consent.reject": "Відхилити",
  "footer": "Футер",
  "footer.next": "Вперед",
  "footer.previous": "Назад",
  "header": "Хедер",
  "meta.comments": "Коментарі",
  "meta.source": "Вихідний код",
  "nav": "Навігація",
  "readtime.one": "Час на прочитання: 1 хвилина",
  "readtime.other": "Час на прочитання: # хвилин",
  "rss.created": "RSS стрічка",
  "rss.updated": "RSS стрічка оновленого контенту",
  "search": "Шукати",
  "search.placeholder": "Пошук",
  "search.share": "Поділитись",
  "search.reset": "Очистити",
  "search.result.initializer": "Пошук розпочато",
  "search.result.placeholder": "Розпочніть писати для пошуку",
  "search.result.none": "Збігів не знайдено",
  "search.result.one": "Знайдено 1 збіг",
  "search.result.other": "Знайдено # збігів",
  "search.result.more.one": "Ще 1 збіг на цій сторінці",
  "search.result.more.other": "Ще # збігів на цій сторінці",
  "search.result.term.missing": "Не знайдено запиту",
  "select.language": "Обрати мову",
  "select.version": "Обрати версію",
  "source": "Перейти до вихідного коду",
  "source.file.contributors": "Контриб'ютори",
  "source.file.date.created": "Створено",
  "source.file.date.updated": "Востаннє оновлено",
  "tabs": "Вкладки",
  "toc": "Зміст",
  "top": "Повернутись нагору"
}[key] }}{% endmacro %}
