{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "fa",
  "direction": "rtl",
  "action.edit": "این صفحه را ویرایش کنید",
  "action.skip": "پرش به محتویات",
  "action.view": "محتویات این صفحه را نشان بده",
  "announce.dismiss": "این را دیگر نشان نده",
  "blog.archive": "بایگانی",
  "blog.categories": "دسته‌بندی‌ها",
  "blog.categories.in": "در",
  "blog.continue": "ادامه به خواندن",
  "blog.draft": "پیش‌نویس",
  "blog.index": "برگشت به فهرست",
  "blog.meta": "فراداده",
  "blog.references": "پیوندهای مربوط",
  "clipboard.copy": "کپی کردن",
  "clipboard.copied": "کپی شد",
  "consent.accept": "تایید",
  "consent.manage": "مدیریت تنظیمات",
  "consent.reject": "رد کردن",
  "footer": "پاورقی",
  "footer.next": "بعدی",
  "footer.previous": "قبلی",
  "header": "سرتیتر",
  "meta.comments": "نظرات",
  "meta.source": "منبع",
  "nav": "هدایت",
  "readtime.one": "1 دقیقه زمان خواندن",
  "readtime.other": "# دقیقه زمان خواندن",
  "rss.created": "خوراک آراس‌اس",
  "rss.updated": "خوراک آراساس محتویات به‌روز شده",
  "search": "جستجو",
  "search.config.pipeline": " ",
  "search.placeholder": "جستجو",
  "search.share": "هم‌رسانی",
  "search.reset": "بازنشانی",
  "search.result.initializer": "راه‌اندازی جستجو",
  "search.result.placeholder": "برای شروع جستجو تایپ کنید",
  "search.result.none": "سندی یافت نشد",
  "search.result.one": "1 سند یافت شد",
  "search.result.other": "# سند یافت شد",
  "search.result.more.one": "1 مورد دیگر در این صفحه",
  "search.result.more.other": "# مورد دیگر در این صفحه",
  "search.result.term.missing": "موجود نیست",
  "select.language": "انتخاب زبان",
  "select.version": "انتخاب ویرایش",
  "source": "رفتن به مخزن",
  "source.file.contributors": "مشارکت کنندگان",
  "source.file.date.created": "ایجاد شده",
  "source.file.date.updated": "اخرین بروزرسانی",
  "tabs": "زبانه‌ها",
  "toc": "فهرست موضوعات",
  "top": "برگشت به بالا"
}[key] }}{% endmacro %}
