{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "is",
  "action.edit": "Breyta þessari síðu",
  "action.skip": "Hoppa yfir í efnið",
  "action.view": "Skoða frumgögn þessarar síðu",
  "announce.dismiss": "Ekki sýna þetta aftur",
  "blog.archive": "Safn",
  "blog.categories": "Flokkar",
  "blog.categories.in": "í",
  "blog.continue": "Lesa meira",
  "blog.draft": "Uppkast",
  "blog.index": "Til baka í yfirlit",
  "blog.meta": "Lýsigögn",
  "blog.references": "Þessu tengt",
  "clipboard.copy": "Afrita á klemmuspjald",
  "clipboard.copied": "Afritað á klemmuspjald",
  "consent.accept": "Samþykkja",
  "consent.manage": "Breyta stillingum",
  "consent.reject": "Hafna",
  "footer": "Síðufótur",
  "footer.next": "Næsta",
  "footer.previous": "Fyrri",
  "header": "Haus",
  "meta.comments": "Umræður",
  "meta.source": "Frumgögn",
  "nav": "Valmynd",
  "readtime.one": "1 mín lestur",
  "readtime.other": "# mín lestur",
  "rss.created": "RSS veita",
  "rss.updated": "RSS veita fyrir uppfært innihald",
  "search": "Leita",
  "search.placeholder": "Leita",
  "search.share": "Deila",
  "search.reset": "Hreinsa",
  "search.result.initializer": "Ræsi leitarvél",
  "search.result.placeholder": "Byrjaðu að skrifa til að hefja leit",
  "search.result.none": "Engar síður fundust",
  "search.result.one": "1 síða fannst",
  "search.result.other": "# síður fundust",
  "search.result.more.one": "1 til viðbótar á þessari síðu",
  "search.result.more.other": "# til viðbótar á þessari síðu",
  "search.result.term.missing": "Vantar",
  "select.language": "Veldu tungumál",
  "select.version": "Veldu útgáfu",
  "source": "Fara í gagnageymslu",
  "source.file.contributors": "Meðhöfundar",
  "source.file.date.created": "Búið til",
  "source.file.date.updated": "Síðast uppfært",
  "tabs": "Flipar",
  "toc": "Efnisyfirlit",
  "top": "Fara aftur efst"
}[key] }}{% endmacro %}
