// Copyright (c) 2025-2026 Zensical and contributors

// SPDX-License-Identifier: MIT
// All contributions are certified under the DCO

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.

// ----------------------------------------------------------------------------

//! HTTP request error.

use std::result;
use thiserror::Error;

use crate::http::{component, Status};

// ----------------------------------------------------------------------------
// Enums
// ----------------------------------------------------------------------------

/// HTTP request error.
#[derive(Debug, Error)]
pub enum Error {
    /// HTTP parser error.
    #[error(transparent)]
    Parser(httparse::Error),

    /// HTTP component error.
    #[error(transparent)]
    Component(#[from] component::Error),

    /// HTTP request incomplete.
    #[error("request incomplete")]
    Incomplete,

    /// HTTP request validation failed.
    #[error("request validation failed: {0}")]
    Validation(Status),
}

// ----------------------------------------------------------------------------
// Trait implementations
// ----------------------------------------------------------------------------

impl From<httparse::Error> for Error {
    /// Creates an error from an HTTP parser error.
    ///
    /// We validate the maximum number of headers with the size of the buffer
    /// that we pass to [`httparse`], so we don't need to check it again.
    #[inline]
    fn from(err: httparse::Error) -> Self {
        if let httparse::Error::TooManyHeaders = err {
            Error::Validation(Status::RequestHeaderFieldsTooLarge)
        } else {
            Error::Parser(err)
        }
    }
}

// ----------------------------------------------------------------------------
// Type aliases
// ----------------------------------------------------------------------------

/// HTTP request result.
pub type Result<T = ()> = result::Result<T, Error>;
