{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "be",
  "direction": "ltr",
  "action.edit": "Правіць старонку",
  "action.skip": "Перайсці да зместа",
  "action.view": "Паглядзець зыходны код старонкі",
  "announce.dismiss": "Больш не паказваць",
  "blog.archive": "Заархіваваць",
  "blog.categories": "Катэгорыі",
  "blog.categories.in": "у",
  "blog.continue": "Працягнуць чытаць",
  "blog.draft": "Чарнавік",
  "blog.index": "Вярнуцца на хатнюю",
  "blog.meta": "Метаданыя",
  "blog.references": "Спасылкі па тэме",
  "clipboard.copy": "Скапіраваць у буфер абмена",
  "clipboard.copied": "Скапіравана ў буфер абмена",
  "consent.accept": "Прыняць",
  "consent.manage": "Кіраваць наладамі",
  "consent.reject": "Адхіліць",
  "footer": "Ніжні калантытул",
  "footer.next": "Наступная",
  "footer.previous": "Папярэдняя",
  "header": "Верхні калантытул",
  "meta.comments": "Каментарыі",
  "meta.source": "Зыходны код",
  "nav": "Навігацыя",
  "readtime.one": "Прачытанне зойме 1 хв",
  "readtime.other": "Прачытанне зойме # хв",
  "rss.created": "RSS стужка",
  "rss.updated": "RSS стужка з абноўленым зместам",
  "search": "Пошук",
  "search.config.lang": "ru",
  "search.placeholder": "Пошук",
  "search.share": "Падзяліцца",
  "search.reset": "Ачысціць",
  "search.result.initializer": "Пачынаем пошук",
  "search.result.placeholder": "Пачніце друкаваць для пошуку",
  "search.result.none": "Нічога ня знойдзена",
  "search.result.one": "Адзін адпаведны дакумент",
  "search.result.other": "Адпаведных дакументаў: #",
  "search.result.more.one": "Яшчэ 1 на гэтай старонцы",
  "search.result.more.other": "Яшчэ # на гэтай старонцы",
  "search.result.term.missing": "Адсутнічае",
  "select.language": "Выберыце мову",
  "select.version": "Выберыце версію",
  "source": "Перайсці ў рэпазітар",
  "source.file.contributors": "Укладальнікі",
  "source.file.date.created": "Створана",
  "source.file.date.updated": "Апошняе абнаўленне",
  "tabs": "Укладкі",
  "toc": "Змест",
  "top": "Вярнуцца да пачатку"
}[key] }}{% endmacro %}
