{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "ar",
  "direction": "rtl",
  "action.edit": "عدل الصفحة",
  "action.skip": "انتقل إلى المحتوى",
  "action.view": "عرض مصدر هذه الصفحة",
  "announce.dismiss": "لا تظهر هذا مرة أخرى",
  "blog.archive": "أرشيف",
  "blog.categories": "فئات",
  "blog.categories.in": "ضمن",
  "blog.continue": "أكمل القراءة",
  "blog.draft": "مسودة",
  "blog.index": "رجوع إلى الفهرس",
  "blog.meta": "بيانات وصفية",
  "blog.references": "روابط ذات علاقة",
  "clipboard.copy": "نسخ إلى الحافظة",
  "clipboard.copied": "تم النسخ الى الحافظة",
  "consent.accept": "قبول",
  "consent.manage": "إدارة الإعدادات",
  "consent.reject": "رفض",
  "footer": "هامش سفلي",
  "footer.next": "التالية",
  "footer.previous": "السابقة",
  "header": "عنوان العارضة",
  "meta.comments": "التعليقات",
  "meta.source": "المصدر",
  "nav": "تصفح",
  "readtime.one": "قراءة لمدة دقيقة",
  "readtime.other": "دقائق قراءة #",
  "rss.created": "ملقم بالخلاصات",
  "rss.updated": "ملقم بالخلاصات المحدثة",
  "search": "إبحث",
  "search.config.pipeline": " ",
  "search.placeholder": "بحث",
  "search.share": "شارك",
  "search.reset": "مسح كلي",
  "search.result.initializer": "بدء البحث",
  "search.result.placeholder": "اكتب لبدء البحث",
  "search.result.none": "لا توجد نتائج",
  "search.result.one": "نتائج البحث مستند واحد",
  "search.result.other": "نتائج البحث # مستندات",
  "search.result.more.one": "أكثر من 1 في هذه الصفحة",
  "search.result.more.other": "أكثر من # في هذه الصفحة",
  "search.result.term.missing": "مفقود",
  "select.language": "إختر اللغة",
  "select.version": "إختر الإصدار",
  "source": "اذهب إلى المصدر",
  "source.file.contributors": "المساهمون",
  "source.file.date.created": "خلقت",
  "source.file.date.updated": "اخر تحديث",
  "tabs": "نوافذ",
  "toc": "جدول المحتويات",
  "top": "عد إلى الأعلى"
}[key] }}{% endmacro %}
