{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "te",
  "action.edit": "ఈ పేజీలో దిద్దుబాట్లు చేయండి",
  "action.skip": "సమాచారానికి వెళ్లండి",
  "action.view": "నేను ఈ పేజీ యొక్క మూలాన్ని చూడాలనుకుంటున్నాను",
  "announce.dismiss": "దీన్ని మళ్లీ చూపవద్దు",
  "blog.archive": "పాత వ్యాసం",
  "blog.categories": "వర్గాలు",
  "blog.categories.in": "లో",
  "blog.continue": "చదవడం కొనసాగించండి",
  "blog.draft": "ప్రారంభ రచన",
  "blog.index": "సూచికకు తిరిగి వెళ్ళు",
  "blog.meta": "సమాచారం గురించి సమాచారం",
  "blog.references": "సంబంధిత సూచనలు",
  "clipboard.copy": "దీనిని అనుకరించు",
  "clipboard.copied": "దీనిని అతికించు",
  "consent.accept": "నేను దీనిని అంగీకరిస్తున్నాను",
  "consent.manage": "ఆకృతీకరణను నిర్వహించండి",
  "consent.reject": "నేను దీనిని తిరస్కరిస్తున్నాను",
  "footer": "అడిటిప్పణి",
  "footer.next": "తదుపరి భాగం",
  "footer.previous": "మునుపటి భాగం",
  "header": "శీర్షిక విభాగం",
  "meta.comments": "అభిప్రాయాలు",
  "meta.source": "మూలం",
  "nav": "మార్గదర్శక పట్టీ",
  "readtime.one": "చదవడానికి ఒక నిమిషం పడుతుంది",
  "readtime.other": "చదవడానికి # నిమిషాలు పడుతుంది",
  "rss.created": "ఆర్ఎస్ఎస్ సేవ",
  "rss.updated": "ఆర్ఎస్ఎస్ సేవ నుండి తాజా నవీకరణ",
  "search": "వెతకండి",
  "search.placeholder": "వెతకండి",
  "search.share": "పంచుకోండి",
  "search.reset": "తుడిచివేయు",
  "search.result.initializer": "శోధనను ప్రారంభిస్తోంది",
  "search.result.placeholder": "రాయడం ద్వారా వెతకడం ప్రారంభించండి",
  "search.result.none": "సరిపోలే పత్రాలు లేవు",
  "search.result.one": "ఒక సరిపోలే పత్రం",
  "search.result.other": "# సరిపోలే పత్రాలు",
  "search.result.more.one": "ఈ పేజీలో మరొకటి",
  "search.result.more.other": "ఈ పేజీలో ఇంకా # ఉన్నాయి",
  "search.result.term.missing": "తప్పిపోయింది",
  "select.language": "భాషను ఎంచుకోండి",
  "select.version": "సంస్కరణను ఎంచుకోండి",
  "source": "భండారానికి వెళ్ళండి",
  "source.file.contributors": "సహకారులు",
  "source.file.date.created": "సృష్టించబడింది",
  "source.file.date.updated": "చివరి నవీకరణ",
  "tabs": "వివిధ కిటికీలు",
  "toc": "విషయ సూచిక",
  "top": "పైకి తిరిగి వెళ్ళు"
}[key] }}{% endmacro %}
