{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "lb",
  "direction": "ltr",
  "action.edit": "D'Säit beaarbechten",
  "action.skip": "Zum Inhalt iwwersprangen",
  "action.view": "Quellcode uweisen",
  "announce.dismiss": "Net erëm uweisen",
  "blog.archive": "Archiv",
  "blog.categories": "Kategorien",
  "blog.categories.in": "an",
  "blog.continue": "Weider liesen",
  "blog.draft": "Skizz",
  "blog.index": "Zeréck zum Index",
  "blog.meta": "Metadaten",
  "blog.references": "Änlech Links",
  "clipboard.copy": "Kopéieren",
  "clipboard.copied": "Kopéiert",
  "consent.accept": "Accept",
  "consent.manage": "Astellungen beaarbechten",
  "consent.reject": "Ofleenen",
  "footer": "Footer",
  "footer.next": "Weider",
  "footer.previous": "Zeréck",
  "header": "Header",
  "meta.comments": "Kommentaren",
  "meta.source": "Quell",
  "nav": "Navigatioun",
  "readtime.one": "1 min Liesedauer",
  "readtime.other": "# min Liesedauer",
  "rss.created": "RSS feed",
  "rss.updated": "RSS feed vun aktualiséiertem Inhalt",
  "search": "Sichen",
  "search.placeholder": "Sichen",
  "search.share": "Deelen",
  "search.reset": "Läschen",
  "search.result.initializer": "D'Sich gëtt initialiséiert",
  "search.result.placeholder": "Schreif fir eppes ze sichen",
  "search.result.none": "Keng zoutreffend Dokumenter",
  "search.result.one": "1 zoutreffend Dokument",
  "search.result.other": "# zoutreffend Dokumenter",
  "search.result.more.one": "1 méi op dëser Säit",
  "search.result.more.other": "# méi op dëser Säit",
  "search.result.term.missing": "Feelend",
  "select.language": "Sprooch auswielen",
  "select.version": "Versioun auswielen",
  "source": "Op den Repository goen",
  "source.file.contributors": "Matwirkender",
  "source.file.date.created": "Erstallt",
  "source.file.date.updated": "Läscht update",
  "tabs": "Tabs",
  "toc": "Inhaltsverzeichnis",
  "top": "Zeréck zum Ufank"
}[key] }}{% endmacro %}
