{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "ro",
  "action.edit": "Editeaza această pagină",
  "action.skip": "Sari la conținut",
  "action.view": "Vezi sursa acestei pagini",
  "announce.dismiss": "Nu mai arăta asta",
  "blog.archive": "Arhivează",
  "blog.categories": "Categorii",
  "blog.categories.in": "în",
  "blog.continue": "Continuă să citești",
  "blog.draft": "Ciornă",
  "blog.index": "Înapoi la index",
  "blog.meta": "Metadata",
  "blog.references": "Link-uri relevante",
  "clipboard.copy": "Copiază în clipboard",
  "clipboard.copied": "Copiat în clipboard",
  "consent.accept": "Accept",
  "consent.manage": "Gestionați setările",
  "consent.reject": "Refuz",
  "footer": "Subsol",
  "footer.next": "Următor",
  "footer.previous": "Anterior",
  "header": "Antet",
  "meta.comments": "Comentarii",
  "meta.source": "Sursă",
  "nav": "Navigație",
  "readtime.one": "1 minut de citit",
  "readtime.other": "# minut de citit",
  "rss.created": "Flux RSS",
  "rss.updated": "Flux RSS cu conținut actualizat",
  "search": "Caută",
  "search.config.lang": "ro",
  "search.placeholder": "Căutare",
  "search.share": "Distribuie",
  "search.reset": "Resetează",
  "search.result.initializer": "Inițializare căutare",
  "search.result.placeholder": "Tastează pentru a începe căutarea",
  "search.result.none": "Nu a fost găsit niciun document",
  "search.result.one": "1 document găsit",
  "search.result.other": "# documente găsite",
  "search.result.more.one": "Încă 1 pe această pagină",
  "search.result.more.other": "Încă # pe această pagină",
  "search.result.term.missing": "Lipsă",
  "select.language": "Selectează limba",
  "select.version": "Selectează versuine",
  "source": "Accesează repository-ul",
  "source.file.contributors": "Contribuitori",
  "source.file.date.created": "Creată",
  "source.file.date.updated": "Ultima actualizare",
  "tabs": "File",
  "toc": "Cuprins",
  "top": "Înapoi sus"
}[key] }}{% endmacro %}
