{#-
  This file was automatically generated - do not edit
-#}
<button type="button" class="md-top md-icon" data-md-component="top" hidden>
  {% set icon = config.theme.icon.top or "material/arrow-up" %}
  {% include ".icons/" ~ icon ~ ".svg" %}
  {{ lang.t("top") }}
</button>
