{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "lv",
  "action.edit": "Rediģēt šo lapu",
  "action.skip": "Pāriet uz saturu",
  "action.view": "Rādīt šīs lapas avotu",
  "announce.dismiss": "Nerādīt vairs",
  "blog.archive": "Arhivēt",
  "blog.categories": "Kategorijas",
  "blog.categories.in": "iekš",
  "blog.continue": "Turpināt lasīt",
  "blog.draft": "Melnraksts",
  "blog.index": "Atpakaļ uz satura rādītāju",
  "blog.meta": "Metadati",
  "blog.references": "Saistītās saites",
  "clipboard.copy": "Kopēt starpliktuvē",
  "clipboard.copied": "Kopēts starpliktuvē",
  "consent.accept": "Pieņemt",
  "consent.manage": "Pārvaldīt iestatījumus",
  "consent.reject": "Noraidīt",
  "footer": "Kājene",
  "footer.next": "Nākamais",
  "footer.previous": "Iepriekšējais",
  "header": "Galvene",
  "meta.comments": "Komentārs",
  "meta.source": "Avots",
  "nav": "Navigācija",
  "readtime.one": "1 min lasīšana",
  "readtime.other": "# min lasīšana",
  "rss.created": "RSS padeve",
  "rss.updated": "RSS padeve atjauninātajam saturam",
  "search": "Meklēt",
  "search.placeholder": "Meklēt",
  "search.share": "Dalīties",
  "search.reset": "Notīrīt",
  "search.result.initializer": "Notiek meklēšanas inicializācija",
  "search.result.placeholder": "Ierakstiet, lai sāktu meklēšanu",
  "search.result.none": "Nav atbilstošu dokumentu",
  "search.result.one": "1 atbilstošs dokuments",
  "search.result.other": "# atbilstoši dokumenti ",
  "search.result.more.one": "1 šajā lapā",
  "search.result.more.other": "# un vairāk šajā lapā",
  "search.result.term.missing": "Trūkstošs",
  "select.language": "Izvēlies valodu",
  "select.version": "Izvēlies versiju",
  "source": "Doties uz repozitoriju",
  "source.file.contributors": "Veidotāji",
  "source.file.date.created": "Izveidots",
  "source.file.date.updated": "Pēdējoreiz atjaunots",
  "tabs": "Cilnes",
  "toc": "Satura rādītājs",
  "top": "Atpakaļ uz augšu"
}[key] }}{% endmacro %}
