{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "bg",
  "action.edit": "Редактирай тази страница",
  "action.skip": "Към съдържанието",
  "action.view": "Виж съдържанието на тази страница",
  "announce.dismiss": "Не показвай повече",
  "blog.archive": "Архив",
  "blog.categories": "Категории",
  "blog.categories.in": "В",
  "blog.continue": "Продължи четенето",
  "blog.draft": "Чернова",
  "blog.index": "Назад към индекса",
  "blog.meta": "Метаданни",
  "blog.references": "Свързани линкове",
  "clipboard.copy": "Копирай",
  "clipboard.copied": "Копирано",
  "consent.accept": "Приеми",
  "consent.manage": "Управление на настойките",
  "consent.reject": "Откажи",
  "footer": "Долен колонтитул",
  "footer.next": "Следваща",
  "footer.previous": "Предишна",
  "header": "Горен колонтитул",
  "meta.comments": "Коментари",
  "meta.source": "Код",
  "nav": "Навигация",
  "readtime.one": "1 мин четено",
  "readtime.other": "# мин четено",
  "rss.created": "RSS публикации",
  "rss.updated": "RSS публикации с актуализирано съдържание",
  "search": "Търси",
  "search.config.lang": "ru",
  "search.placeholder": "Търси",
  "search.share": "Сподели",
  "search.reset": "Изчисти",
  "search.result.initializer": "Инициализирано търсене",
  "search.result.placeholder": "Започнете да пишете, за да търсите",
  "search.result.none": "Няма резултати",
  "search.result.one": "1 резултат",
  "search.result.other": "# резултата",
  "search.result.more.one": "още 1 на тази страница",
  "search.result.more.other": "още # на тази страница",
  "search.result.term.missing": "Липсващо",
  "select.language": "Избери език",
  "select.version": "Избери версия",
  "source": "Към хранилището",
  "source.file.contributors": "Участници",
  "source.file.date.created": "Създаден",
  "source.file.date.updated": "Последна промяна",
  "tabs": "Табове",
  "toc": "Съдържание",
  "top": "Върни се в началото"
}[key] }}{% endmacro %}
