{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "az",
  "action.edit": "Bu səhifəni redaktə eləmək",
  "action.skip": "Məzmuna keçin",
  "action.view": "Bu səhifənin mənbəyinə baxın",
  "announce.dismiss": "Bunu bir daha göstərməyin",
  "blog.archive": "Arxiv",
  "blog.categories": "Kateqoriyalar",
  "blog.continue": "Oxumağa davam etmək",
  "blog.draft": "Qaralama",
  "blog.index": "İndeksə qayıtmaq",
  "blog.meta": "Metadata",
  "blog.references": "Əlaqədar bağlantılar",
  "clipboard.copy": "Mübadilə buferə kopyalayın",
  "clipboard.copied": "Mübadilə buferə kopyalanıb",
  "consent.accept": "Qəbul etmək",
  "consent.manage": "Parametrləri idarə edin",
  "consent.reject": "Qəbul eləməmək",
  "footer.next": "Növbəti",
  "footer.previous": "Əvvəlki",
  "header": "Başlıq",
  "meta.comments": "Şərhlər",
  "meta.source": "Mənbə",
  "nav": "Navigasiya",
  "readtime.one": "1 dəqiqəyə  oxumaq",
  "readtime.other": "# dəqiqəyə oxumaq",
  "rss.created": "RSS lenti",
  "rss.updated": "Yenilənmiş məzmunun RSS lenti",
  "search": "Axtarmaq",
  "search.placeholder": "Axtarmaq",
  "search.share": "Paylaşmaq",
  "search.reset": "Silmək",
  "search.result.initializer": "Axtarış işə salınır",
  "search.result.placeholder": "Axtarmağa başlamaq üçün yazın",
  "search.result.none": "Uyğun sənədlər yoxdur",
  "search.result.one": "1 dənə uyğun nəticə var",
  "search.result.other": "# dənə uyğun nəticə var",
  "search.result.more.one": "Bu səhifədə daha 1 dənə",
  "search.result.more.other": "Bu səhifədə daha # dənə",
  "search.result.term.missing": "İtkin",
  "select.language": "Dili seçmək",
  "select.version": "Versiyanı seçmək",
  "source": "Repositoriyaya baxmaq",
  "source.file.contributors": "Töhfəçilər",
  "source.file.date.created": "Yaradılmışdır",
  "source.file.date.updated": "Son yeniləmə",
  "tabs": "Nişanlar",
  "toc": "Mündəricat",
  "top": "Yuxarıya qayıtmaq"
}[key] }}{% endmacro %}
