{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "ka",
  "action.edit": "გვერდის რედარქირება",
  "action.skip": "კონტენტზე გადასვლა",
  "clipboard.copy": "კოპირება",
  "clipboard.copied": "კოპირებულია",
  "footer.next": "შემდეგი",
  "footer.previous": "წინა",
  "meta.comments": "კომენტარები",
  "meta.source": "წყარო",
  "nav": "ნავიგაცია",
  "search.config.pipeline": " ",
  "search.placeholder": "ძებნა",
  "search.reset": "გასუფთავება",
  "search.result.placeholder": "ჩაწერე ძებნის დასაწყებად",
  "search.result.none": "დოკუმენტი ვერ მოიძებნა",
  "search.result.one": "მოიძებნა 1 დოკუმენტი",
  "search.result.other": "მოიძებნა # დოკუმენტი",
  "search.result.more.one": "კიდევ 1 ამ გვერდზე",
  "search.result.more.other": "კიდევ # ამ გვერდზე",
  "source": "საცავში გადასვლა",
  "source.file.date.created": "შეიქმნა",
  "source.file.date.updated": "ბოლო განახლება",
  "tabs": "ტაბები",
  "toc": "სარჩევი"
}[key] }}{% endmacro %}
