{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "et",
  "action.edit": "Muuda seda lehte",
  "action.skip": "Keri sisuni",
  "action.view": "Vaata lehe lähtekoodi",
  "announce.dismiss": "Ära näita uuesti",
  "blog.archive": "Arhiiv",
  "blog.categories": "Kategooriad",
  "blog.categories.in": "sees",
  "blog.continue": "Jätka lugemist",
  "blog.draft": "Mustand",
  "blog.index": "Tagasi indeksisse",
  "blog.meta": "Metaandmed",
  "blog.references": "Seotud lingid",
  "clipboard.copy": "Kopeeri lõikelauale",
  "clipboard.copied": "Kopeeritud",
  "consent.accept": "Nõustu",
  "consent.manage": "Halda seadeid",
  "consent.reject": "Keeldu",
  "footer": "Jalus",
  "footer.next": "Järgmine",
  "footer.previous": "Eelmine",
  "header": "Päis",
  "meta.comments": "Kommentaarid",
  "meta.source": "Lähtekood",
  "nav": "Navigatsioon",
  "readtime.one": "1-minutiline lugemine",
  "readtime.other": "#-minutiline lugemine",
  "rss.created": "RSS-voog",
  "rss.updated": "Uuendatud sisu RSS-voog",
  "search": "Otsing",
  "search.placeholder": "Otsi",
  "search.share": "Jaga",
  "search.reset": "Lähtesta",
  "search.result.initializer": "Otsingu lähtestamine",
  "search.result.placeholder": "Otsimiseks kirjuta siia",
  "search.result.none": "Otsingule ei leitud ühtegi vastet",
  "search.result.one": "Leiti üks tulemus",
  "search.result.other": "Leiti # tulemust",
  "search.result.more.one": "1 veel sellel lehel",
  "search.result.more.other": "# veel sellel lehel",
  "search.result.term.missing": "Puudub",
  "select.language": "Vali keel",
  "select.version": "Vali versioon",
  "source": "Ava repositooriumis",
  "source.file.contributors": "Autorid",
  "source.file.date.created": "Loodud",
  "source.file.date.updated": "Viimane uuendus",
  "tabs": "Vaheleht",
  "toc": "Sisukord",
  "top": "Üles"
}[key] }}{% endmacro %}
