{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "zh-Hant",
  "action.edit": "編輯此頁",
  "action.skip": "跳轉至",
  "action.view": "查看源代碼",
  "announce.dismiss": "不再顯示此訊息",
  "blog.archive": "存檔",
  "blog.categories": "分類",
  "blog.categories.in": "分類在",
  "blog.continue": "繼續閲讀",
  "blog.draft": "草稿",
  "blog.index": "回到首頁",
  "blog.meta": "元數據",
  "blog.references": "相關鏈接",
  "clipboard.copy": "拷貝",
  "clipboard.copied": "已拷貝",
  "consent.accept": "接受",
  "consent.manage": "管理設置",
  "consent.reject": "拒絕",
  "footer": "頁脚",
  "footer.next": "下一頁",
  "footer.previous": "上一頁",
  "header": "頁首",
  "meta.comments": "評論",
  "meta.source": "來源",
  "search.config.pipeline": "stemmer",
  "search.config.separator": "[\\s\\u200b\\u3000\\-、。，．？！；]+",
  "nav": "導航",
  "readtime.one": "需要 1 分鐘閲讀",
  "readtime.other": "需要 # 分鐘閲讀",
  "rss.created": "簡易資訊聚合",
  "rss.updated": "更新之部分的簡易資訊聚合",
  "search": "搜尋",
  "search.config.pipeline": "stemmer",
  "search.config.separator": "[\\s\\-，。]+",
  "search.placeholder": "搜尋",
  "search.share": "分享",
  "search.reset": "清空",
  "search.result.initializer": "正在初始化搜尋引擎",
  "search.result.placeholder": "鍵入以開始檢索",
  "search.result.none": "沒有找到符合條件的結果",
  "search.result.one": "找到 1 个符合條件的結果",
  "search.result.other": "找到 # 個符合條件的結果",
  "search.result.more.one": "此頁尚有 1 個符合的項目",
  "search.result.more.other": "此頁尚有 # 個符合的項目",
  "search.result.term.missing": "缺失",
  "select.language": "選擇語言",
  "select.version": "選擇版本",
  "source": "前往倉庫",
  "source.file.contributors": "貢獻者",
  "source.file.date.created": "建立日期",
  "source.file.date.updated": "最後更新",
  "tabs": "標籤頁",
  "toc": "目錄",
  "top": "回到頂部"
}[key] }}{% endmacro %}
