{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "sh",
  "action.edit": "Ažuriraj stranicu",
  "action.skip": "Idi na tekst",
  "action.view": "Pogledaj izvorni kod ove stranice",
  "announce.dismiss": "Ne prikazuj mi ovo više",
  "blog.archive": "Arhiva",
  "blog.categories": "Kategorije",
  "blog.categories.in": "u",
  "blog.continue": "Nastavi sa čitanjem",
  "blog.draft": "Nacrt",
  "blog.index": "Nazad na početak",
  "blog.meta": "Metapodaci",
  "blog.references": "Povezani linkovi",
  "clipboard.copy": "Kopiraj u klipbord",
  "clipboard.copied": "Iskopirano u klipbord",
  "consent.accept": "Prihvati",
  "consent.manage": "Promeni podešavanja",
  "consent.reject": "Odbij",
  "footer": "Podnožje",
  "footer.next": "Sledeće",
  "footer.previous": "Prethodno",
  "header": "Zaglavlje",
  "meta.comments": "Komentari",
  "meta.source": "Izvor",
  "nav": "Navigacija",
  "readtime.one": "1 minut čitanja",
  "readtime.other": "# minuta čitanja",
  "rss.created": "RSS kanal",
  "rss.updated": "RSS kanal sa ažuriranim sadržajem",
  "search": "Pretraga",
  "search.placeholder": "Pretraga",
  "search.share": "Deljenje",
  "search.reset": "Očisti",
  "search.result.initializer": "Inicijalizujem pretragu",
  "search.result.placeholder": "Unesite pojam pretrage",
  "search.result.none": "Ništa nije pronađeno",
  "search.result.one": "1 rezultat pretrage",
  "search.result.other": "# rezultata pretrage",
  "search.result.more.one": "još 1 na ovoj strani",
  "search.result.more.other": "još # na ovoj strani",
  "search.result.term.missing": "Nedostaje",
  "select.language": "Izaberi jezik",
  "select.version": "Izaberi verziju",
  "source": "Idi u repozitorijum",
  "source.file.contributors": "Saradnici",
  "source.file.date.created": "Kreiran",
  "source.file.date.updated": "Ažuriran",
  "tabs": "Tabovi",
  "toc": "Sadržaj",
  "top": "Nazad na vrh"
}[key] }}{% endmacro %}
