{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "hy",
  "action.edit": "Խմբագրել այս էջը",
  "action.skip": "Անցնել պարունակությանը",
  "action.view": "Դիտել այս էջի սկզբնաղբյուրը",
  "announce.dismiss": "Այլևս չցուցադրել",
  "blog.archive": "Արխիվ",
  "blog.categories": "Կատեգորիաներ",
  "blog.categories.in": "in",
  "blog.continue": "Շարունակել կարդալ",
  "blog.draft": "Սևագիր",
  "blog.index": "Հետ դեպի ինդեքս",
  "blog.meta": "Մետատվյալներ",
  "blog.references": "Առնչվող հղումներ",
  "clipboard.copy": "Պատճենել",
  "clipboard.copied": "Պատճենված է",
  "consent.accept": "Ընդունել",
  "consent.manage": "Կառավարել կարգավորումները",
  "consent.reject": "Մերժել",
  "footer": "Էջատակ",
  "footer.next": "Հաջորդը",
  "footer.previous": "Նախորդը",
  "header": "Գլխագիր",
  "meta.comments": "Մեկնաբանությունները",
  "meta.source": "Աղբյուր",
  "nav": "Տեղորոշում",
  "readtime.one": "Ընթերցում՝ 1 րոպե",
  "readtime.other": "Ընթերցում՝ # րոպե",
  "rss.created": "RSS հոսք",
  "rss.updated": "Արդիացված բովանդակության RSS հոսք",
  "search": "Որոնում",
  "search.config.pipeline": " ",
  "search.placeholder": "Որոնում",
  "search.share": "Կիսվել",
  "search.reset": "Ջնջել",
  "search.result.initializer": "Որոնում",
  "search.result.placeholder": "Մուտքագրեք որոնելու համար",
  "search.result.none": "Արդյունքներ չկան",
  "search.result.one": "1 արդյունք",
  "search.result.other": "# արդյունք",
  "search.result.more.one": "ևս 1-ը այս էջում",
  "search.result.more.other": "ևս #-ը այս էջում",
  "search.result.term.missing": "Բացակայում է",
  "select.language": "Ընտրել լեզուն",
  "select.version": "Ընտրել տարբերակը",
  "source": "Դեպի պահոց",
  "source.file.contributors": "Հեղինակողներ",
  "source.file.date.created": "Ստեղծված է",
  "source.file.date.updated": "Վերջին թարմացումը",
  "tabs": "Ներդիրներ",
  "toc": "Բովանդակություն",
  "top": "Վերադառնալ սկիզբ"
}[key] }}{% endmacro %}
